namespace SourceGit.Views
{
    public partial class CommitDetailStandalone : ChromelessWindow
    {
        public CommitDetailStandalone()
        {
            InitializeComponent();
        }
    }
}
