namespace SourceGit.Views
{
    public partial class RevisionCompareStandalone : ChromelessWindow
    {
        public RevisionCompareStandalone()
        {
            InitializeComponent();
        }
    }
}
