using Avalonia.Controls;

namespace SourceGit.Views
{
    public partial class RemoveWorktree : UserControl
    {
        public RemoveWorktree()
        {
            InitializeComponent();
        }
    }
}
