namespace SourceGit.Views
{
    public partial class Statistics : ChromelessWindow
    {
        public Statistics()
        {
            CloseOnESC = true;
            InitializeComponent();
        }
    }
}
